'''
Project      : 
FilePath     : \PypiPkg\src\yanhekt\__init__.py
Descripttion : 
Author       : GDDG08
Date         : 2024-04-02 17:54:53
LastEditors  : GDDG08
LastEditTime : 2024-04-02 17:54:59
'''
from .yanhekt import YanHeKT
